#pragma once

#include <base/attached_dataspace.h>
#include <base/lock.h>
#include <base/rpc_client.h>

// gapfruit includes
#include <timestamp_session/timestamp_session.h>

namespace Timestamp {
	using Genode::uint64_t;
	using namespace Genode;

	class Session_client;
}


class Timestamp::Session_client : public Genode::Rpc_client<Session>
{
	private:

		Lock _lock { };

		/**
		 * Shared-memory buffer used for carrying the payload
		 * of read/write operations
		 */
		Attached_dataspace _io_buffer;

	public:
		Session_client(Region_map &local_rm, Capability<Session> cap) :
			Rpc_client<Session>    { cap },
			_io_buffer             { local_rm, call<Rpc_dataspace>() }
		{ }

		uint64_t get() const override
		{
			return call<Rpc_get>();
		}

		uint64_t get(uint8_t const  *hash_buf,
		             size_t          hash_buf_size,
		             uint8_t        *rfc3161_buf,
		             size_t          rfc3161_buf_size,
		             size_t         &rfc3161_length) override
		{
			if ((hash_buf == nullptr) || (hash_buf_size == 0) ||
			    (rfc3161_buf == nullptr) || (rfc3161_buf_size == 0)) {
				throw Invalid_input();
			}

			Lock::Guard _guard(_lock);

			if (hash_buf_size > _io_buffer.size()) throw Buffer_too_small();

			Genode::memcpy(_io_buffer.local_addr<char>(), hash_buf, hash_buf_size);

			uint64_t time_seconds = call<Rpc_get_rfc3161>(hash_buf_size, rfc3161_length);

			if (rfc3161_length > _io_buffer.size()) throw Buffer_too_small();
			if (rfc3161_length > rfc3161_buf_size) throw Buffer_too_small();

			Genode::memcpy(rfc3161_buf, _io_buffer.local_addr<char>(), rfc3161_length);

			return time_seconds;
		}
};

